@@ -30,6 +30,7 @@
 #include "object.h"
 #include <gjs/gjs-module.h>
 #include <gjs/compat.h>
+#include <gjs/runtime.h>
 #include "repo.h"
 #include "proxyutils.h"
 #include "function.h"
@@ -44,7 +45,9 @@ typedef struct {
     GIBoxedInfo *info;
     GType gtype;
     gint zero_args_constructor; /* -1 if none */
+    jsid zero_args_constructor_name;
     gint default_constructor; /* -1 if none */
+    jsid default_constructor_name;
 
     /* instance info */
     void *gboxed; /* NULL if we are the prototype and not an instance */
@@ -116,8 +119,8 @@ gjs_define_static_methods(JSContext    *context,
  */
 static JSBool
 boxed_new_resolve(JSContext *context,
-                  JSObject  *obj,
-                  jsid       id,
+                  JSObject **obj,
+                  jsid      *id,
                   unsigned   flags,
                   JSObject **objp)
 {
@@ -127,11 +130,11 @@ boxed_new_resolve(JSContext *context,
 
     *objp = NULL;
 
-    if (!gjs_get_string_id(context, id, &name))
+    if (!gjs_get_string_id(context, *id, &name))
         return JS_TRUE; /* not resolved, but no error */
 
-    priv = priv_from_js(context, obj);
-    gjs_debug_jsprop(GJS_DEBUG_GBOXED, "Resolve prop '%s' hook obj %p priv %p", name, obj, priv);
+    priv = priv_from_js(context, *obj);
+    gjs_debug_jsprop(GJS_DEBUG_GBOXED, "Resolve prop '%s' hook obj %p priv %p", name, *obj, priv);
 
     if (priv == NULL)
         goto out; /* wrong class */
@@ -159,7 +162,7 @@ boxed_new_resolve(JSContext *context,
                       g_base_info_get_namespace( (GIBaseInfo*) priv->info),
                       g_base_info_get_name( (GIBaseInfo*) priv->info));
 
-            boxed_proto = obj;
+            boxed_proto = *obj;
 
             if (gjs_define_function(context, boxed_proto, priv->gtype,
                                     (GICallableInfo *)method_info) == NULL) {
@@ -301,7 +304,7 @@ boxed_init_from_props(JSContext   *context,
             goto out;
         }
 
-        if (!gjs_object_require_property(context, props, "property list", name, &value)) {
+        if (!gjs_object_require_property(context, props, "property list", prop_id, &value)) {
             g_free(name);
             goto out;
         }
@@ -326,21 +329,24 @@ boxed_init_from_props(JSContext   *context,
 static JSBool
 boxed_invoke_constructor(JSContext   *context,
                          JSObject    *obj,
-                         const gchar *constructor_name,
+                         jsid         constructor_name,
                          unsigned     argc,
                          jsval       *argv,
                          jsval       *rval)
 {
     jsval js_constructor, js_constructor_func;
+    jsid constructor_const;
 
-    if (!gjs_object_require_property (context, obj, NULL, "constructor", &js_constructor))
+    constructor_const = gjs_runtime_get_const_string(JS_GetRuntime(context),
+                                                     GJS_STRING_CONSTRUCTOR);
+    if (!gjs_object_require_property(context, obj, NULL, constructor_const, &js_constructor))
         return JS_FALSE;
 
-    if (!gjs_object_require_property (context, JSVAL_TO_OBJECT(js_constructor), NULL,
-                                      constructor_name, &js_constructor_func))
+    if (!gjs_object_require_property(context, JSVAL_TO_OBJECT(js_constructor), NULL,
+                                     constructor_name, &js_constructor_func))
         return JS_FALSE;
 
-    return gjs_call_function_value (context, NULL, js_constructor_func, argc, argv, rval);
+    return gjs_call_function_value(context, NULL, js_constructor_func, argc, argv, rval);
 }
 
 static JSBool
@@ -352,9 +358,13 @@ boxed_new(JSContext   *context,
           jsval       *rval)
 {
     if (priv->gtype == G_TYPE_VARIANT) {
+        jsid constructor_name;
+
         /* Short-circuit construction for GVariants by calling into the JS packing
            function */
-        return boxed_invoke_constructor (context, obj, "_new_internal", argc, argv, rval);
+        constructor_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
+                                                        GJS_STRING_NEW_INTERNAL);
+        return boxed_invoke_constructor (context, obj, constructor_name, argc, argv, rval);
     }
 
     /* If the structure is registered as a boxed, we can create a new instance by
@@ -390,21 +400,12 @@ boxed_new(JSContext   *context,
     } else if (priv->can_allocate_directly) {
         boxed_new_direct(priv);
     } else if (priv->default_constructor >= 0) {
-        GIFunctionInfo *constructor;
-        const gchar *constructor_name;
         JSBool retval;
 
         /* for simplicity, we simply delegate all the work to the actual JS constructor
            function (which we retrieve from the JS constructor, that is, Namespace.BoxedType,
            or object.constructor, given that object was created with the right prototype */
-
-        constructor = g_struct_info_get_method(priv->info, priv->default_constructor);
-        constructor_name = g_base_info_get_name((GIBaseInfo*)constructor);
-
-        retval = boxed_invoke_constructor(context, obj, constructor_name, argc, argv, rval);
-
-        g_base_info_unref((GIBaseInfo*)constructor);
-
+        retval = boxed_invoke_constructor(context, obj, priv->default_constructor_name, argc, argv, rval);
         return retval;
     } else {
         gjs_throw(context, "Unable to construct struct type %s since it has no default constructor and cannot be allocated directly",
@@ -443,13 +444,13 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(boxed)
     GJS_INC_COUNTER(boxed);
 
     g_assert(priv_from_js(context, object) == NULL);
-    JS_SetPrivate(context, object, priv);
+    JS_SetPrivate(object, priv);
 
     gjs_debug_lifecycle(GJS_DEBUG_GBOXED,
                         "boxed constructor, obj %p priv %p",
                         object, priv);
 
-    proto = JS_GetPrototype(context, object);
+    proto = JS_GetPrototype(object);
     gjs_debug_lifecycle(GJS_DEBUG_GBOXED, "boxed instance __proto__ is %p", proto);
     /* If we're the prototype, then post-construct we'll fill in priv->info.
      * If we are not the prototype, though, then we'll get ->info from the
@@ -507,12 +508,12 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(boxed)
 }
 
 static void
-boxed_finalize(JSContext *context,
-               JSObject  *obj)
+boxed_finalize(JSFreeOp *fop,
+               JSObject *obj)
 {
     Boxed *priv;
 
-    priv = priv_from_js(context, obj);
+    priv = JS_GetPrivate(obj);
     gjs_debug_lifecycle(GJS_DEBUG_GBOXED,
                         "finalize, obj %p priv %p", obj, priv);
     if (priv == NULL)
@@ -592,13 +593,13 @@ get_nested_interface_object (JSContext   *context,
         return JS_FALSE;
     }
 
-    proto = gjs_lookup_boxed_prototype(context, (GIBoxedInfo*) interface_info);
+    proto = gjs_lookup_generic_prototype(context, (GIBoxedInfo*) interface_info);
     proto_priv = priv_from_js(context, proto);
 
     offset = g_field_info_get_offset (field_info);
 
     obj = JS_NewObjectWithGivenProto(context,
-                                     JS_GET_CLASS(context, proto), proto,
+                                     JS_GetClass(proto), proto,
                                      gjs_get_import_global (context));
 
     if (obj == NULL)
@@ -606,7 +607,7 @@ get_nested_interface_object (JSContext   *context,
 
     GJS_INC_COUNTER(boxed);
     priv = g_slice_new0(Boxed);
-    JS_SetPrivate(context, obj, priv);
+    JS_SetPrivate(obj, priv);
     priv->info = (GIBoxedInfo*) interface_info;
     g_base_info_ref( (GIBaseInfo*) priv->info);
     priv->gtype = g_registered_type_info_get_g_type ((GIRegisteredTypeInfo*) interface_info);
@@ -619,7 +620,7 @@ get_nested_interface_object (JSContext   *context,
     /* We never actually read the reserved slot, but we put the parent object
      * into it to hold onto the parent object.
      */
-    JS_SetReservedSlot(context, obj, 0,
+    JS_SetReservedSlot(obj, 0,
                        OBJECT_TO_JSVAL (parent_obj));
 
     *value = OBJECT_TO_JSVAL(obj);
@@ -627,10 +628,10 @@ get_nested_interface_object (JSContext   *context,
 }
 
 static JSBool
-boxed_field_getter (JSContext *context,
-                    JSObject  *obj,
-                    jsid       id,
-                    jsval     *value)
+boxed_field_getter (JSContext            *context,
+                    JSHandleObject        obj,
+                    JSHandleId            id,
+                    JSMutableHandleValue  value)
 {
     Boxed *priv;
     GIFieldInfo *field_info;
@@ -638,11 +639,11 @@ boxed_field_getter (JSContext *context,
     GArgument arg;
     gboolean success = FALSE;
 
-    priv = priv_from_js(context, obj);
+    priv = priv_from_js(context, *obj._);
     if (!priv)
         return JS_FALSE;
 
-    field_info = get_field_info(context, priv, id);
+    field_info = get_field_info(context, priv, *id._);
     if (!field_info)
         return JS_FALSE;
 
@@ -663,9 +664,9 @@ boxed_field_getter (JSContext *context,
         if (g_base_info_get_type (interface_info) == GI_INFO_TYPE_STRUCT ||
             g_base_info_get_type (interface_info) == GI_INFO_TYPE_BOXED) {
 
-            success = get_nested_interface_object (context, obj, priv,
+            success = get_nested_interface_object (context, *obj._, priv,
                                                    field_info, type_info, interface_info,
-                                                   value);
+                                                   value._);
 
             g_base_info_unref ((GIBaseInfo *)interface_info);
 
@@ -682,7 +683,7 @@ boxed_field_getter (JSContext *context,
         goto out;
     }
 
-    if (!gjs_value_from_g_argument (context, value,
+    if (!gjs_value_from_g_argument (context, value._,
                                     type_info,
                                     &arg,
                                     TRUE))
@@ -718,7 +719,7 @@ set_nested_interface_object (JSContext   *context,
         return JS_FALSE;
     }
 
-    proto = gjs_lookup_boxed_prototype(context, (GIBoxedInfo*) interface_info);
+    proto = gjs_lookup_generic_prototype(context, (GIBoxedInfo*) interface_info);
     proto_priv = priv_from_js(context, proto);
 
     /* If we can't directly copy from the source object we need
@@ -807,21 +808,20 @@ out:
 }
 
 static JSBool
-boxed_field_setter (JSContext *context,
-                    JSObject  *obj,
-                    jsid       id,
-                    JSBool     strict,
-                    jsval     *value)
+boxed_field_setter (JSContext            *context,
+                    JSHandleObject        obj,
+                    JSHandleId            id,
+                    JSBool                strict,
+                    JSMutableHandleValue  value)
 {
     Boxed *priv;
     GIFieldInfo *field_info;
     gboolean success = FALSE;
 
-    priv = priv_from_js(context, obj);
+    priv = priv_from_js(context, *obj._);
     if (!priv)
         return JS_FALSE;
-
-    field_info = get_field_info(context, priv, id);
+    field_info = get_field_info(context, priv, *id._);
     if (!field_info)
         return JS_FALSE;
 
@@ -832,7 +832,7 @@ boxed_field_setter (JSContext *context,
         goto out;
     }
 
-    success = boxed_set_field_from_value (context, priv, field_info, *value);
+    success = boxed_set_field_from_value (context, priv, field_info, *value._);
 
 out:
     g_base_info_unref ((GIBaseInfo *)field_info);
@@ -869,8 +869,7 @@ define_boxed_class_fields (JSContext *context,
      * memory overhead.
      */
     if (n_fields > 256) {
-        gjs_debug(GJS_DEBUG_ERROR,
-                  "Only defining the first 256 fields in boxed type '%s'",
+        g_warning("Only defining the first 256 fields in boxed type '%s'",
                   g_base_info_get_name ((GIBaseInfo *)priv->info));
         n_fields = 256;
     }
@@ -941,8 +940,7 @@ static struct JSClass gjs_boxed_class = {
     boxed_finalize,
     NULL,
     NULL,
-    NULL,
-    NULL, NULL, NULL, NULL, NULL
+    NULL, NULL, NULL
 };
 
 static JSPropertySpec gjs_boxed_proto_props[] = {
@@ -950,60 +948,10 @@ static JSPropertySpec gjs_boxed_proto_props[] = {
 };
 
 static JSFunctionSpec gjs_boxed_proto_funcs[] = {
-    { "toString", (JSNative)to_string_func, 0, 0 },
+    { "toString", JSOP_WRAPPER((JSNative)to_string_func), 0, 0 },
     { NULL }
 };
 
-JSObject*
-gjs_lookup_boxed_constructor(JSContext    *context,
-                                GIBoxedInfo  *info)
-{
-    JSObject *ns;
-    JSObject *constructor;
-
-    ns = gjs_lookup_namespace_object(context, (GIBaseInfo*) info);
-
-    if (ns == NULL)
-        return NULL;
-
-    constructor = NULL;
-    if (gjs_define_boxed_class(context, ns, info,
-                                  &constructor, NULL))
-        return constructor;
-    else
-        return NULL;
-}
-
-JSObject*
-gjs_lookup_boxed_prototype(JSContext    *context,
-                           GIBoxedInfo  *info)
-{
-    JSObject *ns;
-    JSObject *proto;
-
-    ns = gjs_lookup_namespace_object(context, (GIBaseInfo*) info);
-
-    if (ns == NULL)
-        return NULL;
-
-    proto = NULL;
-    if (gjs_define_boxed_class(context, ns, info, NULL, &proto))
-        return proto;
-    else
-        return NULL;
-}
-
-JSClass*
-gjs_lookup_boxed_class(JSContext    *context,
-                          GIBoxedInfo  *info)
-{
-    JSObject *prototype;
-
-    prototype = gjs_lookup_boxed_prototype(context, info);
-
-    return JS_GET_CLASS(context, prototype);
-}
-
 static gboolean
 type_can_be_allocated_directly(GITypeInfo *type_info)
 {
@@ -1120,10 +1068,12 @@ struct_is_simple(GIStructInfo *info)
 }
 
 static void
-boxed_fill_prototype_info(Boxed *priv)
+boxed_fill_prototype_info(JSContext *context,
+                          Boxed     *priv)
 {
     int i, n_methods;
     int first_constructor = -1;
+    jsid first_constructor_name = JSID_VOID;
 
     priv->gtype = g_registered_type_info_get_g_type( (GIRegisteredTypeInfo*) priv->info);
     priv->zero_args_constructor = -1;
@@ -1145,34 +1095,49 @@ boxed_fill_prototype_info(Boxed *priv)
 
             flags = g_function_info_get_flags(func_info);
             if ((flags & GI_FUNCTION_IS_CONSTRUCTOR) != 0) {
-                if (first_constructor < 0)
+                if (first_constructor < 0) {
+                    const char *name;
+
+                    name = g_base_info_get_name((GIBaseInfo*) func_info);
                     first_constructor = i;
+                    first_constructor_name = gjs_intern_string_to_id(context, name);
+                }
 
                 if (priv->zero_args_constructor < 0 &&
-                    g_callable_info_get_n_args((GICallableInfo*) func_info) == 0)
+                    g_callable_info_get_n_args((GICallableInfo*) func_info) == 0) {
+                    const char *name;
+
+                    name = g_base_info_get_name((GIBaseInfo*) func_info);
                     priv->zero_args_constructor = i;
+                    priv->zero_args_constructor_name = gjs_intern_string_to_id(context, name);
+                }
 
                 if (priv->default_constructor < 0 &&
-                    strcmp(g_base_info_get_name ((GIBaseInfo*) func_info), "new") == 0)
+                    strcmp(g_base_info_get_name ((GIBaseInfo*) func_info), "new") == 0) {
                     priv->default_constructor = i;
+                    priv->default_constructor_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
+                                                                                  GJS_STRING_NEW);
+                }
             }
 
             g_base_info_unref((GIBaseInfo*) func_info);
         }
 
-        if (priv->default_constructor < 0)
+        if (priv->default_constructor < 0) {
             priv->default_constructor = priv->zero_args_constructor;
-        if (priv->default_constructor < 0)
+            priv->default_constructor_name = priv->zero_args_constructor_name;
+        }
+        if (priv->default_constructor < 0) {
             priv->default_constructor = first_constructor;
+            priv->default_constructor_name = first_constructor_name;
+        }
     }
 }
 
-JSBool
+void
 gjs_define_boxed_class(JSContext    *context,
-                          JSObject     *in_object,
-                          GIBoxedInfo  *info,
-                          JSObject    **constructor_p,
-                          JSObject    **prototype_p)
+                       JSObject     *in_object,
+                       GIBoxedInfo  *info)
 {
     const char *constructor_name;
     JSObject *prototype;
@@ -1187,31 +1152,6 @@ gjs_define_boxed_class(JSContext    *context,
 
     constructor_name = g_base_info_get_name( (GIBaseInfo*) info);
 
-    if (gjs_object_get_property(context, in_object, constructor_name, &value)) {
-        JSObject *constructor;
-
-        if (!JSVAL_IS_OBJECT(value)) {
-            gjs_throw(context, "Existing property '%s' does not look like a constructor",
-                      constructor_name);
-            return JS_FALSE;
-        }
-
-        constructor = JSVAL_TO_OBJECT(value);
-
-        gjs_object_get_property(context, constructor, "prototype", &value);
-        if (!JSVAL_IS_OBJECT(value)) {
-            gjs_throw(context, "boxed %s prototype property does not appear to exist or has wrong type", constructor_name);
-            return JS_FALSE;
-        } else {
-            if (prototype_p)
-                *prototype_p = JSVAL_TO_OBJECT(value);
-            if (constructor_p)
-                *constructor_p = constructor;
-
-            return JS_TRUE;
-        }
-    }
-
     if (!gjs_init_class_dynamic(context, in_object,
                                 NULL, /* parent prototype */
                                 g_base_info_get_namespace( (GIBaseInfo*) info),
@@ -1228,21 +1168,21 @@ gjs_define_boxed_class(JSContext    *context,
                                 NULL,
                                 &prototype,
                                 &constructor)) {
-        gjs_log_exception(context, NULL);
-        gjs_fatal("Can't init class %s", constructor_name);
+        gjs_log_exception(context);
+        g_error("Can't init class %s", constructor_name);
     }
 
     GJS_INC_COUNTER(boxed);
     priv = g_slice_new0(Boxed);
     priv->info = info;
-    boxed_fill_prototype_info(priv);
+    boxed_fill_prototype_info(context, priv);
 
     g_base_info_ref( (GIBaseInfo*) priv->info);
     priv->gtype = g_registered_type_info_get_g_type ((GIRegisteredTypeInfo*) priv->info);
-    JS_SetPrivate(context, prototype, priv);
+    JS_SetPrivate(prototype, priv);
 
     gjs_debug(GJS_DEBUG_GBOXED, "Defined class %s prototype is %p class %p in object %p",
-              constructor_name, prototype, JS_GET_CLASS(context, prototype), in_object);
+              constructor_name, prototype, JS_GetClass(prototype), in_object);
 
     priv->can_allocate_directly = struct_is_simple (priv->info);
 
@@ -1252,14 +1192,6 @@ gjs_define_boxed_class(JSContext    *context,
     value = OBJECT_TO_JSVAL(gjs_gtype_create_gtype_wrapper(context, priv->gtype));
     JS_DefineProperty(context, constructor, "$gtype", value,
                       NULL, NULL, JSPROP_PERMANENT);
-
-    if (constructor_p)
-        *constructor_p = constructor;
-
-    if (prototype_p)
-        *prototype_p = prototype;
-
-    return JS_TRUE;
 }
 
 JSObject*
@@ -1280,11 +1212,11 @@ gjs_boxed_from_c_struct(JSContext             *context,
                       "Wrapping struct %s %p with JSObject",
                       g_base_info_get_name((GIBaseInfo *)info), gboxed);
 
-    proto = gjs_lookup_boxed_prototype(context, info);
+    proto = gjs_lookup_generic_prototype(context, info);
     proto_priv = priv_from_js(context, proto);
 
     obj = JS_NewObjectWithGivenProto(context,
-                                     JS_GET_CLASS(context, proto), proto,
+                                     JS_GetClass(proto), proto,
                                      gjs_get_import_global (context));
 
     GJS_INC_COUNTER(boxed);
@@ -1293,7 +1225,7 @@ gjs_boxed_from_c_struct(JSContext             *context,
     *priv = *proto_priv;
     g_base_info_ref( (GIBaseInfo*) priv->info);
 
-    JS_SetPrivate(context, obj, priv);
+    JS_SetPrivate(obj, priv);
 
     if ((flags & GJS_BOXED_CREATION_NO_COPY) != 0) {
         /* we need to create a JS Boxed which references the
