@@ -30,6 +30,7 @@
 
 #include <gjs/gjs-module.h>
 #include <gjs/compat.h>
+#include <gjs/runtime.h>
 #include <util/log.h>
 
 #include <girepository.h>
@@ -46,12 +47,12 @@ GJS_DEFINE_PRIV_FROM_JS(Interface, gjs_interface_class)
 GJS_NATIVE_CONSTRUCTOR_DEFINE_ABSTRACT(interface)
 
 static void
-interface_finalize(JSContext *context,
-                   JSObject  *obj)
+interface_finalize(JSFreeOp *fop,
+                   JSObject *obj)
 {
     Interface *priv;
 
-    priv = priv_from_js(context, obj);
+    priv = JS_GetPrivate(obj);
 
     if (priv == NULL)
         return;
@@ -100,8 +101,8 @@ gjs_define_static_methods(JSContext       *context,
 
 static JSBool
 interface_new_resolve(JSContext *context,
-                      JSObject  *obj,
-                      jsid       id,
+                      JSObject **obj,
+                      jsid      *id,
                       unsigned   flags,
                       JSObject **objp)
 {
@@ -112,10 +113,10 @@ interface_new_resolve(JSContext *context,
 
     *objp = NULL;
 
-    if (!gjs_get_string_id(context, id, &name))
+    if (!gjs_get_string_id(context, *id, &name))
         return JS_TRUE;
 
-    priv = priv_from_js(context, obj);
+    priv = priv_from_js(context, *obj);
 
     if (priv == NULL)
         goto out;
@@ -123,14 +124,14 @@ interface_new_resolve(JSContext *context,
     method_info = g_interface_info_find_method((GIInterfaceInfo*) priv->info, name);
 
     if (method_info != NULL) {
-        if (gjs_define_function(context, obj,
+        if (gjs_define_function(context, *obj,
                                 priv->gtype,
                                 (GICallableInfo*)method_info) == NULL) {
             g_base_info_unref((GIBaseInfo*)method_info);
             goto out;
         }
 
-        *objp = obj;
+        *objp = *obj;
         g_base_info_unref((GIBaseInfo*)method_info);
     }
 
@@ -155,8 +156,7 @@ static struct JSClass gjs_interface_class = {
     interface_finalize,
     NULL,
     NULL,
-    NULL,
-    NULL, NULL, NULL, NULL, NULL
+    NULL, NULL, NULL
 };
 
 static JSPropertySpec gjs_interface_proto_props[] = {
@@ -170,8 +170,7 @@ static JSFunctionSpec gjs_interface_proto_funcs[] = {
 JSBool
 gjs_define_interface_class(JSContext       *context,
                            JSObject        *in_object,
-                           GIInterfaceInfo *info,
-                           JSObject       **prototype_p)
+                           GIInterfaceInfo *info)
 {
     Interface *priv;
     const char *constructor_name;
@@ -181,32 +180,6 @@ gjs_define_interface_class(JSContext       *context,
 
     constructor_name = g_base_info_get_name((GIBaseInfo*)info);
 
-    gjs_object_get_property(context, in_object, constructor_name, &value);
-    if (!JSVAL_IS_VOID(value)) {
-        JSObject *constructor;
-
-        if (!JSVAL_IS_OBJECT(value)) {
-            gjs_throw(context, "Existing property '%s' does not look like a constructor",
-                      constructor_name);
-            return JS_FALSE;
-        }
-
-        constructor = JSVAL_TO_OBJECT(value);
-
-        gjs_object_get_property(context, constructor, "prototype", &value);
-        if (!JSVAL_IS_OBJECT(value)) {
-            gjs_throw(context, "prototype property does not appear to exist or has wrong type");
-            return JS_FALSE;
-        } else {
-            if (prototype_p)
-                *prototype_p = JSVAL_TO_OBJECT(value);
-
-            return JS_TRUE;
-        }
-
-        return JS_TRUE;
-    }
-
     if (!gjs_init_class_dynamic(context, in_object,
                                 NULL,
                                 g_base_info_get_namespace((GIBaseInfo*)info),
@@ -223,7 +196,7 @@ gjs_define_interface_class(JSContext       *context,
                                 NULL,
                                 &prototype,
                                 &constructor)) {
-        gjs_fatal("Can't init class %s", constructor_name);
+        g_error("Can't init class %s", constructor_name);
     }
 
     GJS_INC_COUNTER(interface);
@@ -231,7 +204,7 @@ gjs_define_interface_class(JSContext       *context,
     priv->info = info;
     priv->gtype = g_registered_type_info_get_g_type(priv->info);
     g_base_info_ref((GIBaseInfo*)priv->info);
-    JS_SetPrivate(context, prototype, priv);
+    JS_SetPrivate(prototype, priv);
 
     gjs_define_static_methods(context, constructor, priv->gtype, priv->info);
 
@@ -239,8 +212,5 @@ gjs_define_interface_class(JSContext       *context,
     JS_DefineProperty(context, constructor, "$gtype", value,
                       NULL, NULL, JSPROP_PERMANENT);
 
-    if (prototype_p)
-        *prototype_p = prototype;
-
     return JS_TRUE;
 }
