@@ -37,6 +37,7 @@
 #include "gerror.h"
 
 #include <gjs/compat.h>
+#include <gjs/runtime.h>
 
 #include <util/log.h>
 #include <util/misc.h>
@@ -44,8 +45,6 @@
 #include <girepository.h>
 #include <string.h>
 
-#define DUMPBIN "_gjs_private"
-
 typedef struct {
     void *dummy;
 
@@ -55,15 +54,17 @@ static struct JSClass gjs_repo_class;
 
 GJS_DEFINE_PRIV_FROM_JS(Repo, gjs_repo_class)
 
-static JSObject * lookup_override_function(JSContext *, const char *);
+static JSObject * lookup_override_function(JSContext *, jsid);
 
 static JSObject*
 resolve_namespace_object(JSContext  *context,
                          JSObject   *repo_obj,
+                         jsid        ns_id,
                          const char *ns_name)
 {
     GIRepository *repo;
     GError *error;
+    jsid versions_name;
     jsval versions_val;
     JSObject *versions;
     jsval version_val;
@@ -74,7 +75,9 @@ resolve_namespace_object(JSContext  *context,
 
     JS_BeginRequest(context);
 
-    if (!gjs_object_require_property(context, repo_obj, "GI repository object", "versions", &versions_val) ||
+    versions_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
+                                                 GJS_STRING_GI_VERSIONS);
+    if (!gjs_object_require_property(context, repo_obj, "GI repository object", versions_name, &versions_val) ||
         !JSVAL_IS_OBJECT(versions_val)) {
         gjs_throw(context, "No 'versions' property in GI repository object");
 
@@ -85,7 +88,7 @@ resolve_namespace_object(JSContext  *context,
     versions = JSVAL_TO_OBJECT(versions_val);
 
     version = NULL;
-    if (JS_GetProperty(context, versions, ns_name, &version_val) &&
+    if (JS_GetPropertyById(context, versions, ns_id, &version_val) &&
         JSVAL_IS_STRING(version_val)) {
         gjs_string_to_utf8(context, version_val, &version);
     }
@@ -119,9 +122,9 @@ resolve_namespace_object(JSContext  *context,
                            ns_name, OBJECT_TO_JSVAL(namespace),
                            NULL, NULL,
                            GJS_MODULE_PROP_FLAGS))
-        gjs_fatal("no memory to define ns property");
+        g_error("no memory to define ns property");
 
-    override = lookup_override_function(context, ns_name);
+    override = lookup_override_function(context, ns_id);
     if (override && !JS_CallFunctionValue (context,
                                            namespace, /* thisp */
                                            OBJECT_TO_JSVAL(override), /* callee */
@@ -156,8 +159,8 @@ resolve_namespace_object(JSContext  *context,
  */
 static JSBool
 repo_new_resolve(JSContext *context,
-                 JSObject  *obj,
-                 jsid       id,
+                 JSObject **obj,
+                 jsid      *id,
                  unsigned   flags,
                  JSObject **objp)
 {
@@ -167,7 +170,7 @@ repo_new_resolve(JSContext *context,
 
     *objp = NULL;
 
-    if (!gjs_get_string_id(context, id, &name))
+    if (!gjs_get_string_id(context, *id, &name))
         return JS_TRUE; /* not resolved, but no error */
 
     /* let Object.prototype resolve these */
@@ -175,17 +178,17 @@ repo_new_resolve(JSContext *context,
         strcmp(name, "toString") == 0)
         goto out;
 
-    priv = priv_from_js(context, obj);
+    priv = priv_from_js(context, *obj);
     gjs_debug_jsprop(GJS_DEBUG_GREPO, "Resolve prop '%s' hook obj %p priv %p", name, obj, priv);
 
     if (priv == NULL) /* we are the prototype, or have the wrong class */
         goto out;
 
     JS_BeginRequest(context);
-    if (resolve_namespace_object(context, obj, name) == NULL) {
+    if (resolve_namespace_object(context, *obj, *id, name) == NULL) {
         ret = JS_FALSE;
     } else {
-        *objp = obj; /* store the object we defined the prop in */
+        *objp = *obj; /* store the object we defined the prop in */
     }
     JS_EndRequest(context);
 
@@ -197,12 +200,12 @@ repo_new_resolve(JSContext *context,
 GJS_NATIVE_CONSTRUCTOR_DEFINE_ABSTRACT(repo)
 
 static void
-repo_finalize(JSContext *context,
-              JSObject  *obj)
+repo_finalize(JSFreeOp *fop,
+              JSObject *obj)
 {
     Repo *priv;
 
-    priv = priv_from_js(context, obj);
+    priv = JS_GetPrivate(obj);
     gjs_debug_lifecycle(GJS_DEBUG_GREPO,
                         "finalize, obj %p priv %p", obj, priv);
     if (priv == NULL)
@@ -246,10 +249,15 @@ repo_new(JSContext *context)
     JSObject *repo;
     JSObject *global;
     JSObject *versions;
+    JSObject *private_ns;
+    JSBool found;
+    jsid versions_name, private_ns_name;
 
     global = gjs_get_import_global(context);
 
-    if (!gjs_object_has_property(context, global, gjs_repo_class.name)) {
+    if (!JS_HasProperty(context, global, gjs_repo_class.name, &found))
+        return NULL;
+    if (!found) {
         JSObject *prototype;
         prototype = JS_InitClass(context, global,
                                  /* parent prototype JSObject* for
@@ -274,9 +282,7 @@ repo_new(JSContext *context)
                                  /* funcs of constructor, MyConstructor.myfunc() */
                                  NULL);
         if (prototype == NULL)
-            gjs_fatal("Can't init class %s", gjs_repo_class.name);
-
-        g_assert(gjs_object_has_property(context, global, gjs_repo_class.name));
+            g_error("Can't init class %s", gjs_repo_class.name);
 
         gjs_debug(GJS_DEBUG_GREPO, "Initialized class %s prototype %p",
                   gjs_repo_class.name, prototype);
@@ -293,22 +299,27 @@ repo_new(JSContext *context)
     GJS_INC_COUNTER(repo);
 
     g_assert(priv_from_js(context, repo) == NULL);
-    JS_SetPrivate(context, repo, priv);
+    JS_SetPrivate(repo, priv);
 
     gjs_debug_lifecycle(GJS_DEBUG_GREPO,
                         "repo constructor, obj %p priv %p", repo, priv);
 
     versions = JS_NewObject(context, NULL, NULL, global);
+    versions_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
+                                                 GJS_STRING_GI_VERSIONS);
+    JS_DefinePropertyById(context, repo,
+                          versions_name,
+                          OBJECT_TO_JSVAL(versions),
+                          NULL, NULL,
+                          JSPROP_PERMANENT);
 
-    JS_DefineProperty(context, repo,
-                      "versions",
-                      OBJECT_TO_JSVAL(versions),
-                      NULL, NULL,
-                      JSPROP_PERMANENT);
-
-    g_assert(gjs_object_has_property(context, repo, "versions"));
-
-    JS_DefineObject(context, repo, DUMPBIN, NULL, NULL, JSPROP_PERMANENT);
+    private_ns = JS_NewObject(context, NULL, NULL, global);
+    private_ns_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
+                                                   GJS_STRING_PRIVATE_NS_MARKER);
+    JS_DefinePropertyById(context, repo,
+                          private_ns_name,
+                          OBJECT_TO_JSVAL(private_ns),
+                          NULL, NULL, JSPROP_PERMANENT);
 
     /* FIXME - hack to make namespaces load, since
      * gobject-introspection does not yet search a path properly.
@@ -459,11 +470,9 @@ gjs_define_info(JSContext  *context,
             gtype = g_registered_type_info_get_g_type((GIRegisteredTypeInfo*)info);
 
             if (g_type_is_a (gtype, G_TYPE_PARAM)) {
-                if (!gjs_define_param_class(context, in_object, NULL))
-                    return JS_FALSE;
+                gjs_define_param_class(context, in_object);
             } else if (g_type_is_a (gtype, G_TYPE_OBJECT)) {
-                if (!gjs_define_object_class(context, in_object, gtype, NULL, NULL))
-                    return JS_FALSE;
+                gjs_define_object_class(context, in_object, (GIObjectInfo*) info, gtype, NULL);
             } else {
                 gjs_throw (context,
                            "Unsupported type %s, deriving from fundamental %s",
@@ -474,23 +483,22 @@ gjs_define_info(JSContext  *context,
         break;
     case GI_INFO_TYPE_STRUCT:
     case GI_INFO_TYPE_BOXED:
-        if (!gjs_define_boxed_class(context, in_object, (GIBoxedInfo*) info, NULL, NULL))
-            return JS_FALSE;
+        gjs_define_boxed_class(context, in_object, (GIBoxedInfo*) info);
         break;
     case GI_INFO_TYPE_UNION:
-        if (!gjs_define_union_class(context, in_object, (GIUnionInfo*) info, NULL, NULL))
+        if (!gjs_define_union_class(context, in_object, (GIUnionInfo*) info))
             return JS_FALSE;
         break;
     case GI_INFO_TYPE_ENUM:
         if (g_enum_info_get_error_domain((GIEnumInfo*) info)) {
             /* define as GError subclass */
-            if (!gjs_define_error_class(context, in_object, (GIEnumInfo*) info, NULL, NULL))
-                return JS_FALSE;
+            gjs_define_error_class(context, in_object, (GIEnumInfo*) info);
+            break;
         }
         /* fall through */
 
     case GI_INFO_TYPE_FLAGS:
-        if (!gjs_define_enumeration(context, in_object, (GIEnumInfo*) info, NULL))
+        if (!gjs_define_enumeration(context, in_object, (GIEnumInfo*) info))
             return JS_FALSE;
         break;
     case GI_INFO_TYPE_CONSTANT:
@@ -498,8 +506,7 @@ gjs_define_info(JSContext  *context,
             return JS_FALSE;
         break;
     case GI_INFO_TYPE_INTERFACE:
-        if (!gjs_define_interface_class(context, in_object, (GIInterfaceInfo*) info, NULL))
-            return JS_FALSE;
+        gjs_define_interface_class(context, in_object, (GIInterfaceInfo*) info);
         break;
     default:
         gjs_throw(context, "API of type %s not implemented, cannot define %s.%s",
@@ -516,7 +523,11 @@ gjs_define_info(JSContext  *context,
 JSObject*
 gjs_lookup_private_namespace(JSContext *context)
 {
-    return gjs_lookup_namespace_object_by_name(context, DUMPBIN);
+    jsid ns_name;
+
+    ns_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
+                                           GJS_STRING_PRIVATE_NS_MARKER);
+    return gjs_lookup_namespace_object_by_name(context, ns_name);
 }
 
 /* Get the namespace object that the GIBaseInfo should be inside */
@@ -525,6 +536,7 @@ gjs_lookup_namespace_object(JSContext  *context,
                             GIBaseInfo *info)
 {
     const char *ns;
+    jsid ns_name;
 
     ns = g_base_info_get_namespace(info);
     if (ns == NULL) {
@@ -535,40 +547,42 @@ gjs_lookup_namespace_object(JSContext  *context,
         return NULL;
     }
 
-    return gjs_lookup_namespace_object_by_name(context, ns);
+    ns_name = gjs_intern_string_to_id(context, ns);
+    return gjs_lookup_namespace_object_by_name(context, ns_name);
 }
 
 static JSObject*
 lookup_override_function(JSContext  *context,
-                         const char *ns)
+                         jsid        ns_name)
 {
-    JSObject *global;
     jsval importer;
     jsval overridespkg;
     jsval module;
     jsval function;
+    jsid overrides_name, object_init_name;
 
     JS_BeginRequest(context);
-    global = gjs_get_import_global(context);
 
-    importer = JSVAL_VOID;
-    if (!gjs_object_require_property(context, global, "global object", "imports", &importer) ||
-        !JSVAL_IS_OBJECT(importer))
-        goto fail;
+    importer = gjs_get_global_slot(context, GJS_GLOBAL_SLOT_IMPORTS);
+    g_assert(JSVAL_IS_OBJECT(importer));
 
     overridespkg = JSVAL_VOID;
+    overrides_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
+                                                  GJS_STRING_GI_OVERRIDES);
     if (!gjs_object_require_property(context, JSVAL_TO_OBJECT(importer), "importer",
-                                        "overrides", &overridespkg) ||
+                                     overrides_name, &overridespkg) ||
         !JSVAL_IS_OBJECT(overridespkg))
         goto fail;
 
     module = JSVAL_VOID;
-    if (!gjs_object_require_property(context, JSVAL_TO_OBJECT(overridespkg), "GI repository object", ns, &module)
+    if (!gjs_object_require_property(context, JSVAL_TO_OBJECT(overridespkg), "GI repository object", ns_name, &module)
         || !JSVAL_IS_OBJECT(module))
         goto fail;
 
+    object_init_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
+                                                    GJS_STRING_GOBJECT_INIT);
     if (!gjs_object_require_property(context, JSVAL_TO_OBJECT(module), "override module",
-                                     "_init", &function) ||
+                                     object_init_name, &function) ||
         !JSVAL_IS_OBJECT(function))
         goto fail;
 
@@ -583,47 +597,43 @@ lookup_override_function(JSContext  *context,
 
 JSObject*
 gjs_lookup_namespace_object_by_name(JSContext      *context,
-                                    const char     *ns)
+                                    jsid            ns_name)
 {
-    JSObject *global;
     JSObject *repo_obj;
     jsval importer;
     jsval girepository;
     jsval ns_obj;
-
-    /* This is a little bit of a hack, we hardcode an assumption that
-     * the only repo object that exists is called "imports.gi" and is
-     * is stored in the "import global" for the runtime.
-     */
+    jsid gi_name;
 
     JS_BeginRequest(context);
-    global = gjs_get_import_global(context);
 
-    importer = JSVAL_VOID;
-    if (!gjs_object_require_property(context, global, "global object", "imports", &importer) ||
-        !JSVAL_IS_OBJECT(importer)) {
-        gjs_log_exception(context, NULL);
-        gjs_throw(context, "No imports property in global object");
-        goto fail;
-    }
+    importer = gjs_get_global_slot(context, GJS_GLOBAL_SLOT_IMPORTS);
+    g_assert(JSVAL_IS_OBJECT(importer));
 
     girepository = JSVAL_VOID;
+    gi_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
+                                           GJS_STRING_GI_MODULE);
     if (!gjs_object_require_property(context, JSVAL_TO_OBJECT(importer), "importer",
-                                        "gi", &girepository) ||
+                                     gi_name, &girepository) ||
         !JSVAL_IS_OBJECT(girepository)) {
-        gjs_log_exception(context, NULL);
+        gjs_log_exception(context);
         gjs_throw(context, "No gi property in importer");
         goto fail;
     }
 
     repo_obj = JSVAL_TO_OBJECT(girepository);
 
-    if (!gjs_object_require_property(context, repo_obj, "GI repository object", ns, &ns_obj)) {
+    if (!gjs_object_require_property(context, repo_obj, "GI repository object", ns_name, &ns_obj)) {
         goto fail;
     }
 
     if (!JSVAL_IS_OBJECT(ns_obj)) {
-        gjs_throw(context, "Namespace '%s' is not an object?", ns);
+        char *name;
+
+        gjs_get_string_id(context, ns_name, &name);
+        gjs_throw(context, "Namespace '%s' is not an object?", name);
+
+        g_free(name);
         goto fail;
     }
 
@@ -731,3 +741,37 @@ gjs_hyphen_from_camel(const char *camel_name)
 
     return g_string_free(s, FALSE);
 }
+
+JSObject *
+gjs_lookup_generic_prototype(JSContext  *context,
+                             GIBaseInfo *info)
+{
+    JSObject *in_object;
+    JSObject *constructor;
+    const char *constructor_name;
+    jsval value;
+
+    in_object = gjs_lookup_namespace_object(context, (GIBaseInfo*) info);
+    constructor_name = g_base_info_get_name((GIBaseInfo*) info);
+
+    if (G_UNLIKELY (!in_object))
+        return NULL;
+
+    if (!JS_GetProperty(context, in_object, constructor_name, &value))
+        return NULL;
+
+    if (G_UNLIKELY (!JSVAL_IS_OBJECT(value) || JSVAL_IS_NULL(value)))
+        return NULL;
+
+    constructor = JSVAL_TO_OBJECT(value);
+    g_assert(constructor != NULL);
+
+    if (!gjs_object_get_property_const(context, constructor,
+                                       GJS_STRING_PROTOTYPE, &value))
+        return NULL;
+
+    if (G_UNLIKELY (!JSVAL_IS_OBJECT(value)))
+        return NULL;
+
+    return JSVAL_TO_OBJECT(value);
+}
