@@ -160,13 +160,14 @@ gjs_log_error(JSContext *context,
     JSExceptionState *exc_state;
     JSString *jstr;
 
-    JS_BeginRequest(context);
-
     if ((argc != 1 && argc != 2) ||
         !JSVAL_IS_OBJECT (argv[0])) {
         gjs_throw(context, "Must pass an exception and optionally a message to logError()");
+        return JS_FALSE;
     }
 
+    JS_BeginRequest(context);
+
     if (argc == 2) {
         /* JS_ValueToString might throw, in which we will only
          *log that the value could be converted to string */
@@ -570,6 +571,7 @@ gjs_context_constructor (GType                  type,
     js_context = GJS_CONTEXT(object);
 
     js_context->runtime = JS_NewRuntime(32*1024*1024 /* max bytes */);
+    JS_SetNativeStackQuota(js_context->runtime, 1024*1024);
     if (js_context->runtime == NULL)
         g_error("Failed to create javascript runtime");
     JS_SetGCParameter(js_context->runtime, JSGC_MAX_BYTES, 0xffffffff);
@@ -591,12 +593,10 @@ gjs_context_constructor (GType                  type,
      */
     options_flags = JSOPTION_DONT_REPORT_UNCAUGHT | JSOPTION_STRICT | JSOPTION_ALLOW_XML;
 
-#ifdef JSOPTION_JIT
     if (!g_getenv("GJS_DISABLE_JIT")) {
         gjs_debug(GJS_DEBUG_CONTEXT, "Enabling JIT");
-        options_flags |= JSOPTION_JIT;
+        options_flags |= JSOPTION_METHODJIT;
     }
-#endif
 
     JS_SetOptions(js_context->context,
                   JS_GetOptions(js_context->context) | options_flags);
